#!/usr/local/bin/perl

BEGIN {
  my $path=$0;
  $path=~s#[^/\\]+$##;
  # Go to script home in order to read all stuff there
  chdir $path if -d $path;
  # unshift @INC,$path;
}

use strict;
use Tk 800;
use FontDialog;
use Tk::DialogBox;
use Tk::Photo;
use Tk::Table;
use Tk::LabFrame;
use Tk::LabEntry;
use Tk::Checkbutton;
use Tk::Scale;
use Tk::Menubutton;
use Tk::Radiobutton;
use Tk::Canvas;
use Tk::Balloon;
#use diagnostics;

if (grep /^--?h/,@ARGV)
{
  # for plain perl script only!
  system "perldoc $0";
  exit 1;
}

my $ver=q$Revision: 1.7 $;

my @font_options=('-family','-size','-weight','-slant','-underline','-overstrike');
# must be a sequenced set

my %m_eng = (setup_title=>'Arithmetic setup',Accept=>'Accept',Cancel=>'Cancel',
  game_tun=>'Game rules tuning',user_intr=>'User interface',
  distr=>'Maximal numbers played:',logfile=>'Log-file',name=>'Arithmetic for children ;-)',
  log_fsel=>'Select file to store log info',back=>'<< Back',Correct=>'Correct',
  incorrect=>'Incorrect! Must be',Result=>'Result',
  Level=>"Difficulty level\ncontrol possible answers number",
  game_1=>"Game number one:\n Counting articles, getting familiar with numbers",
  game_2=>"Game number two:\n Adding articles, learn plus sign",
  game_3=>"Game number three:\n Comparing articles, learn comparison signs",
  game_4=>"Game number four:\n Substructing articles, learn minus sign",
  game_5=>"Game number five:\n Adding numbers",
  game_6=>"Game number six:\n Comparing numbers",
  game_7=>"Game number seven:\n Substructing numbers",
  game_8=>"Game number eight:\n Random game from any previous",
  ctrl_about=>"About button\nShows 'About' message:\nprogram name, author, version",
  ctrl_exit=>"Exit button\nPress here if you want to leave the game",
  ctrl_setup=>"Setup button\nGrown-ups can change game interface\nand control maximal number used in problems",
  ctrl_nav=>"Start tournament\nAfter pressing this button\nall games will run one-by-one"
  );
my %m_rus = (setup_title=>' -  ',Accept=>'',Cancel=>'',
  game_tun=>'  ',user_intr=>' ',
  distr=>'   :',logfile=>' ',name=>'   ;-)',
  log_fsel=>'    ',back=>'<< ',Correct=>'',
  incorrect=>'!  ',Result=>'',
  Level=>" \n   ",
  game_1=>" :\n  ,   ",
  game_2=>" :\n  ,    ",
  game_3=>" :\n  ,    ",
  game_4=>" :\n  ,    ",
  game_5=>" :\n  ",
  game_6=>" :\n  ",
  game_7=>" :\n  ",
  game_8=>" :\n    ",
  ctrl_about=>" \n   ,    ",
  ctrl_exit=>"\n     ",
  ctrl_setup=>"\n    \n     ",
  ctrl_nav=>" \n   \n      "
  );
foreach (keys %m_rus)
{
  $m_rus{$_} =~ s//chr(0xFF)/eg; # known bug in Russian code page
  # interpretation - last character 'ya' became unprintable in literals
}
my %msg = %m_eng;

my $scale=9; # maximal counted number
# =================== main window section ===============
#  _____________
# %_____________-_x
#|                 | \
#| ()___           |  |
#|     _()         |  |
#|  __()           |   > game_frame
#| ()_             |  |
#|    ()____()     |  |
#|_________________| /
#| <?>  <#>  <!>   | \_  ctrl_frame
#|_________________| /

my $mw = MainWindow->new(-title=>$ver,-background=>'gray');
$mw->resizable(0,0);

my %game_cfg=&read_cfg(
  -user_font=>'', 
  -language=>'English', 
  -logfile=>'arithmetic.log',
  -scale=>9);

$game_cfg{-scale}=4 if $game_cfg{-scale}<4;
$game_cfg{-scale}=9 if $game_cfg{-scale}>9;

if($game_cfg{-language} eq 'English') { %msg = %m_eng; }
else { %msg = %m_rus; }
$scale=$game_cfg{-scale};
$mw->configure(-title=>"$msg{name} $ver");

# reset all games budget:
my (@game_score)=(0,0,0,0,0,0,0,0); # score in freestyle play
my $game_n=1;                       # game number in tournament play
my $button=0;                       # last game exit status (abort if non-zero)
my $tournament=0;                   # tournament game flag

# Picture objects initialization
my %pic;
foreach (qw/bad_guy bad_guy2 c00 c01 c02 c03 c04 c05 c06 c07 c08 c09 c10 c11 c12 c13 c14 c15 c16 c17 c18 c19 c20 c21 d01 d02 d03 d04 d05 d06 d07 d08 d09 plus minus spider exit setup about emerald1 emerald2 papa_smu brainy smurfett cmp0 cmp1 cmp2 navigate/)
{
  $pic{$_}=$mw->Photo(-file=>"resource/$_.gif");
}
$pic{''}=$mw->Photo();

my $game_frame=$mw->Frame()->pack(-side=>'top');
my $ctrl_frame=$mw->Frame()->pack(-side=>'bottom',-fill=>'x',-expand=>1);
$mw->bind("<Escape>", [sub {exit}]);
$mw->bind("<Return>", [sub {exit}]);
$mw->geometry('+120+1');

# ==========
# game_frame
# ==========

my $c=$game_frame->Canvas(-background=>'gray')->pack;

my $game_count=8;
my $cs2=12;

# Game path skeleton
my (@game_coord)=([120,290],[195,251],[35,213],[185,174],[45,135],[165,97],[85,58],[113,20]);

my ($m_w,$m_h)=(0,0);
my ($i,@ovals);

foreach $i (1 .. $game_count)
{
  my ($x,$y)=(($game_coord[$i-1])->[0],($game_coord[$i-1])->[1]);
  $x+=int(3-rand(6)); $x*=1.5; ($game_coord[$i-1])->[0]=$x;
  $y*=1.5; ($game_coord[$i-1])->[1]=$y;

  $m_w=$x if $x>$m_w;
  $m_h=$y if $y>$m_h;
}
my (@smooth_line);
my ($px,$py)=(0,0);
foreach $i (1 .. $game_count)
{
  my ($x,$y)=(($game_coord[$i-1])->[0],($game_coord[$i-1])->[1]);
  if($px || $py)
  {
    # calculate mid-point
    push(@smooth_line,($x+$px)/2,($y+$py)/2);
  }
  push(@smooth_line,$x,$y+$cs2*2,$x,$y,$x,$y-$cs2*2);
  $px=$x; $py=$y;
}
#             _______ top-point
#            /\
#           /  \
#          /     \
#        /         \
#      /             \
# 1. top-point of mount = 
# $game_coord[7])->[0],($game_coord[7])->[1]
# 2. bottom-point =
# $game_coord[4])->[0],($game_coord[4])->[1]
my @mount=
(
  &curved_line(20,(($game_coord[4])->[1]+($game_coord[3])->[1])/2,
  ($game_coord[7])->[0],($game_coord[7])->[1],8),
  &curved_line(($game_coord[7])->[0],($game_coord[7])->[1]
   ,$m_w+20,(($game_coord[4])->[1]+($game_coord[3])->[1])/2,8)
);
# Additional deviations for hill-side lines:
my ($mc_x,$mc_y)=(($mount[0]+$mount[-4])/2,$mount[1]);
foreach $i (0 .. 4)
{
  my $m_p=int(rand($#mount/2-4)+2)*2;
  next if abs($#mount-$m_p*2) < 5;
  my($x0,$y0,$x1,$y1)=($mount[$m_p],$mount[$m_p+1],$mount[$m_p+2],$mount[$m_p+3]);
  splice(@mount,$m_p,2,($x0+$x1)/2,($y0+$y1+$mc_y)/3);
}
$c->createPolygon(@mount,-fill=>'darkgray');
pop(@smooth_line); pop(@smooth_line);
$c->createLine(@smooth_line,-width=>6,-fill=>'blue',-smooth=>1);

my $bb = $c->Balloon(-background=>'lightyellow');

my @status_labels=();
foreach $i (1 .. $game_count)
{
  my ($x,$y)=(($game_coord[$i-1])->[0],($game_coord[$i-1])->[1]);
  my $item = 
    $c->createOval($x-$cs2,$y-$cs2, $x+$cs2, $y+$cs2, -outline => 'black',-fill=>'lightgray');
  $c->addtag('circle_s','withtag',$item); # generalization tag
  $c->addtag("game_$i",'withtag',$item);  # identification tag

  push(@status_labels,$c->Label()->place(-x=>$m_w+30,-y=>$y,-anchor=>'w') );
  &update_status($i-1);
}
my $animation=0;
my $enable_animation=1;

$c->bind('circle_s','<Enter>' => [sub{ $enable_animation=1; shift->itemconfigure(@_) },'current',-fill=>'green']);
$c->bind('circle_s','<Leave>' => [sub{&game_over_check;shift->itemconfigure(@_)},'current',-fill=>'lightgray']);
$c->bind('circle_s','<Button-1>' => [sub{
    return if $animation;
    $enable_animation=0;
    shift->itemconfigure(@_);
    &enter_game($c->gettags('current'));
  },'current',-fill=>'red']);

$c->configure(-width=>($m_w+80+$cs2*2),-height=>($m_h+10+$cs2*2)); # resize canvas

my $radio_frame=$c->Frame()->place(-x=>2,-y=>$m_h,-anchor=>'sw');

my $game_level=1;
my @level_img=qw/smurfett brainy papa_smu/;
foreach (0 .. 2) {
  $radio_frame->Radiobutton(
      -variable => \$game_level,
      -relief   => 'flat',
      -value    => $_+1
    )->pack(-side=>'bottom',-anchor=>'sw');
   $radio_frame->Label(-image=>$pic{$level_img[$_]}
     )->pack(-side=>'bottom',-anchor=>'se');
}

# ==========
# ctrl_frame
# ==========
my $ctrl_about=
  $ctrl_frame->Button(-image=>$pic{'about'},-command=>\&about) ->pack(-side=>'left',-expand=>1);
my $ctrl_setup=
  $ctrl_frame->Button(-image=>$pic{'setup'},-command=>\&setup) ->pack(-side=>'left',-expand=>1);
my $ctrl_exit=
  $ctrl_frame->Button(-image=>$pic{'exit'},-command=>[sub {exit}]) ->pack(-side=>'left',-expand=>1);
my $ctrl_nav=
  $ctrl_frame->Button(-image=>$pic{'navigate'},-command=>[sub {$tournament=1;&next_game}]) ->pack(-side=>'left',-expand=>1);

&attach_balloons();

# ============ more setup =============
my $fd=$mw->FontDialog(-nicefont => 1,-font=>'arial');

# Create log-file
my $expr=0; # last incorrect expression
my ($store_log,$log_file)=&init_log(0,$game_cfg{-logfile});

&apply_font($mw);

if (grep /^[-\/]-?a/,@ARGV) { &about(); }
if (grep /^[-\/]-?[sc]/,@ARGV) { &setup(); }

&about(1600) unless (grep /^[-\/]-?a/,@ARGV);

MainLoop;

# =============== Subroutines ================
#
sub curved_line
{
  my ($x0,$y0,$x1,$y1,$n)=@_;
  my ($i,$x,$y);
  my (@r)=();
  push(@r,$x0,$y0);
  foreach $i(1 .. $n-1)
  {
    $x=$x0+($x1-$x0)/$n*$i+rand(10)-5;
    $y=$y0+($y1-$y0)/$n*$i+rand(10)-5;
    push(@r,$x,$y);
  }
  push(@r,$x1,$y1);
  return @r;
}

# re-open log-file
sub init_log{
  my ($store_log,$log_file)=(@_);
  close LOG if $store_log;
  $store_log=1 if open(LOG,">$log_file");
  return ($store_log,$log_file);
}

# font name convertion
sub font2widget{
  my ($w,$font_desc)=@_;
  return '' unless $font_desc;
  my @font_desc=split(',',$font_desc);
  my %font_cfg=();
  my $i;
  for ($i=0;$i<=$#font_options;$i++)
  {
    $font_cfg{$font_options[$i]}=$font_desc[$i];
  }
  return $w->fontCreate(%font_cfg);
}

sub widget2font{
  my ($w,$font)=@_;
  return '' unless $font;
  my %font_cfg=$w->fontConfigure($font);
  my @font_desc=();
  foreach (@font_options)
  {
    push(@font_desc,$font_cfg{$_});
  }
  return join(',',@font_desc);
}

sub attach_balloons
{
  $bb->attach($radio_frame,-balloonposition => 'mouse',-balloonmsg=>$msg{'Level'});
  $bb->attach($c,-balloonposition => 'mouse',
   -msg => {
     'game_1'=>$msg{'game_1'}, 
     'game_2'=>$msg{'game_2'}, 
     'game_3'=>$msg{'game_3'}, 
     'game_4'=>$msg{'game_4'}, 
     'game_5'=>$msg{'game_5'}, 
     'game_6'=>$msg{'game_6'}, 
     'game_7'=>$msg{'game_7'}, 
     'game_8'=>$msg{'game_8'}});
  $bb->attach($ctrl_about,-balloonposition => 'mouse',-balloonmsg=>$msg{'ctrl_about'});
  $bb->attach($ctrl_setup,-balloonposition => 'mouse',-balloonmsg=>$msg{'ctrl_setup'});
  $bb->attach($ctrl_exit, -balloonposition => 'mouse',-balloonmsg=>$msg{'ctrl_exit'});
  $bb->attach($ctrl_nav,  -balloonposition => 'mouse',-balloonmsg=>$msg{'ctrl_nav'});
}

# config-file read/write
sub read_cfg{
  my %cfg=@_;
  if (open(CFG,'arithmetic.cfg'))
  {
    my @f=<CFG>; close CFG;
    chomp @f; map( {undef $_ if m/^[\s!#\/]/} @f);
    foreach (@f)
    {
      chomp;
      my ($k,$v) = (m/^(-\w+)[\s=>]+([^\s=>][^=>]*)$/);
      $v=&font2widget($mw,$v) if($k eq '-user_font');
      $cfg{$k}=$v if (exists $cfg{$k});
    }
  }
  return %cfg;
}

sub store_cfg{
  my %cfg=@_;
  if (open(CFG,'>arithmetic.cfg'))
  {
    print CFG "#       This configuration file\n".
      "# was automatically generated by 'arithmetic' game.\n".
      "# You can modify any field (on your own risk).\n";
    foreach (keys %cfg)
    {
      next unless $cfg{$_};
      if(/user_font/)
      {
         $cfg{$_}=&widget2font($mw,$cfg{$_});
      }
      print CFG "$_ => $cfg{$_}\n";
    }
    close CFG;
  }
}

sub apply_font{
  my ($widget)=@_;
  $widget->RefontTree(-font => $game_cfg{-user_font}) if $game_cfg{-user_font};
}

# Random picture name generation
sub GenerateBitmapName
{
  my ($count_name)=sprintf("c%02.2d",int(rand(21)));
  return $pic{$count_name};
}

# From game level and correct answer generate possible answers array
sub gen_possible_answers
{
  my ($x)=shift;
  my (@used_answers);
  if($game_level<=2)
  {
    my $from=$x-1; 
    $from=1 if $from<=0; 
    $from=$scale-$game_level-1 if $from+$game_level+1 >= $scale;
    foreach (0 .. $game_level+1)
    {
      push(@used_answers,($from+$_));
    }
  }
  elsif($game_level==3)
  {
    # game_level 3:
    @used_answers=(1 .. $scale);
  }
  foreach (0 .. $#used_answers+2)
  {
    push(@used_answers,splice(@used_answers,rand($#used_answers),1));
  }
  return @used_answers;
}

# Games decoder ;-)
sub game2{ return &arith_game('+',&GenerateBitmapName()); }
sub game3{ return &compare_game(&GenerateBitmapName()); }
sub game4{ return &arith_game('-',&GenerateBitmapName()); }
sub game5{ return &arith_game('+',0); }
sub game6{ return &compare_game(0); }
sub game7{ return &arith_game('-',0); }

sub game1
{
  my $x=int (rand()*($scale-1)+1);
  my $w = $mw->DialogBox( 
    -title=>"-=<(* $game_n *)>=-",
    -buttons => [$msg{'back'}] );
  my $f=$w->Frame()->pack(-side=>'top');
  my $bitmap=&GenerateBitmapName();
  $expr=('@' x $x)." =\n";
  foreach (1 .. $x)
  {
    $f->Label(-image => $bitmap) -> pack(-side=>'left',-ipadx=>3,-anchor=>'nw');
  }
  my $fb=$w->Frame()->pack(-side=>'top');

  $w->OnDestroy(sub{$w->after(10,\&next_game)});

  foreach (&gen_possible_answers($x))
  {
    $fb->Button(-image=>$pic{"d0$_"},-command=>[ \&report, $w, 
      ($_==$x)?"$msg{Correct}\\$_":"$_\\$msg{incorrect}\\$x", 
      ($_==$x)?'info':'error',$game_n])
      ->pack(-side=>'left',-anchor=>'nw');
  }
  $w->resizable(0,0);
  &apply_font($w);
  $button = 0;
  $button = $w->Show;
  &report($w,"$expr $x",'error',$game_n) if ($expr);
  return $button;
}

sub arith_game
{
  my ($op,$bitmap)=(@_);
  my (%op_file)=('-'=>'minus','+'=>'plus');
  my $x=int (rand()*($scale-1)+2);
  my $x1=int (rand()*($x-1)+1);
  my $x2=$x-$x1;
  my $w = $mw->DialogBox( -title=>"-=<(* $game_n *)>=-",
    -buttons => [$msg{'back'}] );

  $w->OnDestroy(sub{$w->after(10,\&next_game)});

  my $f=$w->Frame()->pack(-side=>'top');
  my($d1,$d2,$d)=($x1,$x2,$x);
  ($d1,$d2,$d)=($x,$x1,$x2) if($op eq '-');
  if($bitmap)
  {
    $expr=('@' x $d1)." $op ".('@' x $d2)." =\n";
  }
  else
  {
    $expr="$d1 $op $d2 =\n";
  }
  if($bitmap)
  {
    foreach (1 .. $d1)
    { $f->Label(-image => $bitmap) 
      -> pack(-side=>'left',-ipadx=>3,-fill=>'y',-anchor=>'nw');  }
  }
  else
  {
    $f->Label(-image => $pic{"d0$d1"}) 
    -> pack(-side=>'left',-fill=>'y',-anchor=>'nw');
  }
  $f->Label(-image=>$pic{$op_file{$op}})->pack(-side=>'left',-ipadx=>3);
  if($bitmap)
  {
    foreach (1 .. $d2)
    { $f->Label(-image => $bitmap) 
      -> pack(-side=>'left',-ipadx=>3,-fill=>'y',-anchor=>'nw');  }
  }
  else
  {
    $f->Label(-image => $pic{"d0$d2"}) -> pack(-side=>'left',-fill=>'y',-anchor=>'nw');
  }
  my $fb=$w->Frame()->pack(-side=>'top');
  foreach (&gen_possible_answers($d))
  {
    $fb->Button(-image=>$pic{"d0$_"},-command=>[ \&report, $w, 
      ($_==$d)?"$msg{Correct}\\$_":"$_\\$msg{incorrect}\\$d", ($_==$d)?'info':'error',$game_n])
      ->pack(-side=>'left',-anchor=>'nw');
  }
  $w->resizable(0,0);
  &apply_font($w);
  $button = 0;
  $button=$w->Show;
  &report($w,"$expr $d",'error',$game_n) if ($expr);
  return $button;
}

sub compare_game
{
  my ($bitmap)=(@_);
  my $x0=int (rand()*($scale-1)+2);
  my $x1=int (rand()*($scale-1)+2);
  my $a=($x0<=>$x1)+1;

  my ($po1,$po2,$po3,$po4)=('top','top','x','x');
  ($po1,$po2,$po3,$po4)=('left','left','y','y') if($game_level==2);
  ($po1,$po2,$po3,$po4)=('top','left','x','y')  if($game_level==3);
  my $w = $mw->DialogBox( -title=>"-=<(* $game_n *)>=-",
    -buttons => [$msg{'back'}] );

  $w->OnDestroy(sub{$w->after(10,\&next_game)});

  my $l_fr=$w->Frame()->pack(-side=>'left',-fill=>'y');
  my $c_fr=$w->Frame()->pack(-side=>'left',-fill=>'y',-ipadx=>10);
  my $r_fr=$w->Frame()->pack(-side=>'right',-fill=>'y');

  # show value1
  if($bitmap)
  {
    foreach (1 .. $x0)
    { $l_fr->Label(-image=>$bitmap)
        ->pack(-side=>$po1,-ipadx=>3,-fill=>$po3,-anchor=>'nw',-expand=>1);  }
  }
  else
  {
    $l_fr->Label(-image => $pic{"d0$x0"}) 
    -> pack(-side=>'left',-fill=>'y',-anchor=>'nw');
  }
  # show signs
  my $right_sign=('<','=','>')[$a];
  if($bitmap)
  {
    $expr=('@' x $x0)." ? ".('@' x $x1)."\n";
  }
  else
  {
    $expr="$x0 ? $x1\n";
  }
  foreach (0 .. 2)
  {
    my $sign=('<','=','>')[$_];
    $c_fr->Button(-image=>$pic{"cmp$_"},-command=>[ \&report, $w, 
      ($_==$a)?"$msg{Correct}\\$sign":"$sign\\$msg{incorrect}\\$right_sign", 
      ($_==$a)?'info':'error',$game_n])
      ->pack(-ipady=>3,-expand=>1);
  }
  # show value2
  if($bitmap)
  {
    foreach (1 .. $x1)
    { $r_fr->Label(-image=>$bitmap)
      ->pack(-side=>$po2,-ipadx=>3,-fill=>$po4,-anchor=>'nw',-expand=>1);  }
  }
  else
  {
    $r_fr->Label(-image => $pic{"d0$x1"}) 
    -> pack(-side=>'left',-fill=>'y',-anchor=>'nw');
  }
  
  $w->resizable(0,0);
  &apply_font($w);
  $button = 0;
  $button=$w->Show;
  if ($expr)
  {
    $expr=~s/\?/$right_sign/;
    &report($w,$expr,'error',$game_n);
  }
  return $button;
}

# entering game No $game_n
sub enter_game{
  return if $animation;
  my ($arg)=grep(!/circle_s/,@_);
  my @games=(\&game1,\&game2,\&game3,\&game4,\&game5,\&game6,\&game7);
  ($game_n)=($arg =~ /game_(\d)/);
  my ($real_game_n)=$game_n;
  $real_game_n = int(1+rand($game_count-2)) if($game_n==$game_count);
  # ^^^ generate random game at last stage
  my $p=$games[$real_game_n-1];
  return &$p($game_n);
}

# here we get in after each round
sub next_game
{
  # 0. return immediately if freestyle game (not tournament);
  return unless $tournament;
  return if $animation;
  # 1. return to main window if pressed '<<' button
  if ($button){$tournament=0; $button=0; return;}
  foreach (1 .. $#game_score+2)
  {
    $game_n=$_;
    last if($game_score[$_-1]<2);
  }
  if($game_n >= 1 && $game_n <= $game_count)
  {
    &enter_game("game_$game_n");
  }
  # for game number = 9 - report finish and return to main
  else
  {
    $enable_animation=1 if $game_n>$game_count;
    $game_n=1;$tournament=0;&game_over_check();$enable_animation=0;
    return;
  }
}

# Tell user was the answer correct or not
sub report
{
   my ($w,$msg,$icon,$game_num) = (@_);
   $game_num--;
   $msg=~s/\n/ /g;
   my $db=$w->DialogBox(-title=>$msg{'Result'},-buttons=>['Ok']);
   my $image=$pic{($icon eq 'info')?'bad_guy2':'bad_guy'};
   $db->Label(-image=>$image)->pack(-ipadx=>40,-ipady=>2);
   foreach (split (/\\/,$msg))
   {
     $db->Label(-text=>$_)->pack(-ipadx=>40,-ipady=>2);
   }
   $db->resizable(0,0);
   &apply_font($db);
   $db->Show();
   # calculate game score
   if($icon eq 'info')
   {
     $game_score[$game_num]++ if $game_score[$game_num]<=1;
   }
   else
   {
     $game_score[$game_num]-- if $game_score[$game_num]>=-1;
   }
   # update game status on board
   &update_status($game_num);
   if($store_log)
   {
     print LOG $expr unless $icon eq 'info';
   }
   $expr=0;
   $w->destroy();
   return;
}

# Update status according to game_num
sub update_status
{
  my $game_num=shift;
  $status_labels[$game_num]->configure(-image=>
    $pic{('bad_guy','bad_guy2','','emerald1','emerald2')[$game_score[$game_num]+2]},
    -background=>'gray');
  # game score counted from -2 to 2, so for normal indexing we need <+2>
}

# Check, is there 'gamover' condition?
sub game_over_check
{
  my ($i);
  return unless $enable_animation;
  foreach $i(0 .. $#game_score)
  {
    return unless $game_score[$i]==2;
  }
  $animation=1;
  $mw->Busy();
  # The game is over!!!
  foreach (0 .. 8)
  {
    foreach $i(0 .. $#game_score)
    { Tk::DoOneEvent(0);$mw->after(22);$status_labels[$#game_score-$i]->configure(-image=>$pic{'emerald1'} ); }
    $mw->after(80);
    foreach $i(0 .. $#game_score)
    { Tk::DoOneEvent(0);$mw->after(22);$status_labels[$#game_score-$i]->configure(-image=>$pic{'emerald2'} ); }
    $mw->after(80);
  }
  $mw->Unbusy();
  $animation=0;
}

# Splash screen
sub about
{
  my ($time)=(@_);
  return if $animation;
  my $w = $mw->DialogBox( -title=>$msg{name}, -buttons => ["OK"]);

  $w->Label(-text=>"$msg{name}\n$ver")->pack();
  $w->Label(-image=>$pic{'spider'})->pack();
  $w->Label(-text=>"Copyright (c) 2002 Felix Liberman\n\n".
    "e-mail: FelixL\@Rambler.RU\n\n".
    "IDE: GVIM 6.0")->pack();
  $w->resizable(0,0);
  &apply_font($w);
  $w->bind("<Escape>", sub {$w->bind("<Escape>");$w->destroy()});
  $w->after($time, sub {$w->destroy() if Exists($w)}) if $time;
  $w->Show;
}

sub setup
{
  # draw setup window:
  # +---------------------+-----------------+
  # | Setup:              | Game options:
  # | <FONT...> [example] | (-)....=.....(+)
  # | <Language...> [LNG] |
  # | <Log-file...>       |
  # | [path/name]         |
  # |  <Accept> <Cancel>  |
  # +---------------------+-----------------+
  return if $animation;
  my $w = $mw->Dialog( -title=>$msg{'setup_title'}, 
    -buttons=>[$msg{'Accept'},$msg{'Cancel'}], -default_button=>$msg{'Accept'} );
  $w->Label( -text=> $msg{'setup_title'} )->pack(-ipady=>5);

  # save all setup state
  my %user_cfg=%game_cfg;
  my $lf1=$w->LabFrame(-label => $msg{'user_intr'}, -labelside => "acrosstop")
    ->pack(-side=>'top',-fill=>'x');
  my $lf2=$w->LabFrame(-label => $msg{'game_tun'}, -labelside => "acrosstop")
    ->pack(-side=>'bottom',-fill=>'x');
  my $pf=$lf2->Frame()->pack(-side=>'top',-fill=>'x');
  my $sc_length=($game_cfg{-user_font})?$w->fontMeasure($game_cfg{-user_font},'2' x 50):288;
  $pf->Scale(-orient=>'horizontal',-length=>$sc_length,-from=>4,-to=>9,-tickinterval=>1,-showvalue=>1,
  -variable=>\($user_cfg{-scale}),-label=>$msg{'distr'})->pack(-side=>'bottom');

  my ($f1)=$lf1->Frame()->pack(-side=>'top',-fill=>'x',-ipady=>5);
  my $flbl=$f1->Label(-text=>&widget2font($mw,$user_cfg{-user_font}))
      ->pack(-fill=>'y',-side=>'right',-anchor=>'e');
  my $b=$f1->Button(-text=>'Font',-command=> 
      sub{ 
	my $font=$fd->Show; 
          if($font)
	  {
            $user_cfg{-user_font}=$font; 
	    $flbl->configure(-text=>&widget2font($mw,$font));
	    $flbl->configure(-font=>$font);
          }
        }
    );
  $b->pack(-side=>'left');

  my ($f2)=$lf1->Frame()->pack(-side=>'top',-fill=>'x',-ipady=>5);
  my $lngb=$f2->Menubutton(-underline=>0,-relief=>'raised',-direction=>'below',-text=>'Language')
    ->pack(-side=>'left');
  my $menu = $lngb->menu(-tearoff=>0);
  $lngb->configure(-menu => $menu);
  $lngb->command(-label => "English", -command =>
        sub {$user_cfg{'-language'}='English'});
  $lngb->command(-label => "Russian", -command =>
         sub {$user_cfg{'-language'}='Russian'});

  $f2->Label(-textvariable=>\($user_cfg{'-language'}))->pack(-fill=>'y',-side=>'right',-anchor=>'e');

  my ($f3)=$lf1->Frame()->pack(-side=>'top',-fill=>'x',-ipady=>5);
  my $lfl=$f3->Label(-textvariable=>\($user_cfg{'-logfile'}));
  my $lfb=$f3->Button(-text=>$msg{'logfile'},-command=>
    sub{
	my @types = ( ["Log-files",'.log'], ["All files", '*'] );
	my $file = $user_cfg{'-logfile'};
	$file=~s#.*[/\\]([^/\\]+)$#$1#;
        if($ENV{OS}=~/(^win)|(^$)/i)
        {
	  $file = $w->getSaveFile(-filetypes => \@types,
				-initialfile => $file,
				-defaultextension => '.log',
			        -title=>$msg{'log_fsel'});
        }
        else
        {
          $file = $w->FileSelect(-directory => '.',
                                -initialfile => $file,
                                -title=>$msg{'log_fsel'})->Show;
        }
        if (defined $file and $file ne '') {
	  $user_cfg{'-logfile'}=$file;
        }
    }
  );
  $lfb->pack(-side=>'left');
  $lfl->pack(-fill=>'y',-side=>'right',-anchor=>'e');

  $w->resizable(0,0);
  &apply_font($w);
  # get user reply
  my ($reply)=$w->Show;
  # assign user choise if 'Accept' selected
  if ($reply eq $msg{'Accept'})
  {
    $mw->fontDelete($game_cfg{-user_font}) 
      if($game_cfg{-user_font} && ($game_cfg{-user_font} ne $user_cfg{-user_font}));
    %game_cfg=%user_cfg;
    # apply changes:
    $scale=$game_cfg{-scale};
    # 1. font
    &apply_font($mw);
    # 2. language
    if($game_cfg{-language} eq 'English') { %msg = %m_eng; }
    else { %msg = %m_rus; }
    $mw->configure(-title=>"$msg{name} $ver");
    # baloons:
    &attach_balloons();
    # 3. reopen log-file
    $log_file=$game_cfg{'-logfile'};
    ($store_log,$log_file)=&init_log($store_log,$log_file);
    # 4. And finally - store config itself!
    &store_cfg(%game_cfg);
  }

}

__END__

=head1 NAME

 Arithmetic_II - game for pre-school and elementary school

=head1 SYNOPSIS

 Arithmetic_II [-setup] [-about] [-help]

   -s[etup] - run GUI setup before game start
   -a[bout] - show GUI splash screen before game start
   -h[elp]  - show this help

=head1 DESCRIPTION

 Game have 8 stages with 3 levels
 
 Maximal counted number can be configured from 4 to 9
 
 Currently supported 2 languages: English(default) & Russian(Win CP1251)

 2 main modes: Freestyle(default) and Step-by-step

=head1 REVISION

 $Header: D:\\USR\\FELIX\\SRC\\PERL\\ptk\\arithmetic_II\\RCS\\arithmetic_II.pl,v 1.7 2002/02/05 21:15:16 user Exp user $
 $Revision: 1.7 $
 $Locker: user $
 $Log: arithmetic_II.pl,v $
 Revision 1.1  2002/02/05 21:15:16  user
 Initial revision


=head1 AUTHOR

 Original author: Felix Liberman
 E-mail: felixl@rambler.ru
 WWW: http://Felix.da.ru
 This revision $Author: user $
 
